﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Web;
using System.Web.Security;
using CM.Core;
using CM.Core.Data;
using CM.Core.Model;
using Core.Diagnostics;

namespace CM.Core.Api.MetaWeblog
{
    /// <summary>
    /// HTTP Handler for MetaWeblog API
    /// </summary>
    internal class MetaWeblogHandler : IHttpHandler
    {
        #region IHttpHandler Members

        /// <summary>
        /// Gets a value indicating whether another request can use the <see cref="T:System.Web.IHttpHandler"></see> instance.
        /// </summary>
        /// <value></value>
        /// <returns>true if the <see cref="T:System.Web.IHttpHandler"></see> instance is reusable; otherwise, false.</returns>
        public bool IsReusable
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Process the HTTP Request.  Create XMLRPC request, find method call, process it and create response object and sent it back.
        /// This is the heart of the MetaWeblog API
        /// </summary>
        /// <param name="context"></param>
        public void ProcessRequest(HttpContext context)
        {
            try
            {
                string rootUrl = CommonUtils.AbsoluteWebRoot.ToString();// context.Request.Url.ToString().Substring(0, context.Request.Url.ToString().IndexOf("metaweblog.axd"));
                XMLRPCRequest input = new XMLRPCRequest(context);
                XMLRPCResponse output = new XMLRPCResponse(input.MethodName);

                SysTrace.Trace("MetaWeblog Method: " + input.MethodName);

                switch (input.MethodName)
                {
                    case "metaWeblog.newPost":
                        output.PostID = NewPost(input.BlogID, input.UserName, input.Password, input.Post, input.Publish);
                        break;
                    case "metaWeblog.editPost":
                        output.Completed = EditPost(input.PostID, input.UserName, input.Password, input.Post, input.Publish);
                        break;
                    case "metaWeblog.getPost":
                        output.Post = GetPost(input.PostID, input.UserName, input.Password);
                        break;
                    case "metaWeblog.newMediaObject":
                        output.MediaInfo = NewMediaObject(input.BlogID, input.UserName, input.Password, input.MediaObject, context);
                        break;
                    case "metaWeblog.getCategories":
                        output.Categories = GetCategories(input.BlogID, input.UserName, input.Password, rootUrl);
                        break;
                    case "metaWeblog.getRecentPosts":
                        output.Posts = GetRecentPosts(input.BlogID, input.UserName, input.Password, input.NumberOfPosts);
                        break;
                    case "blogger.getUsersBlogs":
                    case "metaWeblog.getUsersBlogs":
                        output.Blogs = GetUserBlogs(input.AppKey, input.UserName, input.Password, rootUrl);
                        break;
                    case "blogger.deletePost":
                        output.Completed = DeletePost(input.AppKey, input.PostID, input.UserName, input.Password, input.Publish);
                        break;
                    case "blogger.getUserInfo":
                        //Not implemented.  Not planned.
                        throw new MetaWeblogException("10", "The method GetUserInfo is not implemented.");
                    case "wp.newPage":
                        output.PageID = NewPage(input.BlogID, input.UserName, input.Password, input.Page, input.Publish);
                        break;
                    case "wp.getPageList":
                    case "wp.getPages":
                        output.Pages = GetPages(input.BlogID, input.UserName, input.Password);
                        break;
                    case "wp.getPage":
                        output.Page = GetPage(input.BlogID, input.PageID, input.UserName, input.Password);
                        break;
                    case "wp.editPage":
                        output.Completed = EditPage(input.BlogID, input.PageID, input.UserName, input.Password, input.Page, input.Publish);
                        break;
                    case "wp.deletePage":
                        output.Completed = DeletePage(input.BlogID, input.PageID, input.UserName, input.Password);
                        break;
                    case "wp.getAuthors":
                        output.Authors = GetAuthors(input.BlogID, input.UserName, input.Password);
                        break;
                    case "wp.getTags":
                        output.Keywords = GetKeywords(input.BlogID, input.UserName, input.Password);
                        break;
                }

                output.Response(context);
            }
            catch (MetaWeblogException mex)
            {
                XMLRPCResponse output = new XMLRPCResponse("fault");
                MWAFault fault = new MWAFault();
                fault.faultCode = mex.Code;
                fault.faultString = mex.Message;
                output.Fault = fault;
                output.Response(context);
                LogUtils.LogException(mex, System.Diagnostics.EventLogEntryType.Error);
            }
            catch (Exception ex)
            {
                XMLRPCResponse output = new XMLRPCResponse("fault");
                MWAFault fault = new MWAFault();
                fault.faultCode = "0";
                fault.faultString = ex.Message;
                output.Fault = fault;
                output.Response(context);
                LogUtils.LogException(ex, System.Diagnostics.EventLogEntryType.Error);
            }
        }

        #endregion

        #region API Methods

        /// <summary>
        /// metaWeblog.newPost
        /// </summary>
        /// <param name="blogID">always 1000 in BlogEngine since it is a singlar blog instance</param>
        /// <param name="userName">login username</param>
        /// <param name="password">login password</param>
        /// <param name="sentPost">struct with post details</param>
        /// <param name="publish">mark as published?</param>
        /// <returns>postID as string</returns>
        internal string NewPost(string blogID, string userName, string password, MWAPost sentPost, bool publish)
        {
            ValidateRequest(userName, password);

            ItemPost post = ItemPost.NewItemPost();
            User user;
            User author;
            UserService userService = new UserService();
            user = userService.GetByUserLogon(userName);
            author = userService.GetByDisplayName(sentPost.wp_author);

            if (author != null)
            {
                //post.Author = author.DisplayName;
                // Setting AuthorSlug will set author;
                //post.AuthorSlug = author.Slug; 
                post.Author = author;
            }
            if (user != null)
            {
                post.UpdateUser = user.Slug;
            }
            post.Title = sentPost.title;
            post.Content = sentPost.description;
            post.Visible = publish;  // IsPublished
            post.Slug = sentPost.wp_slug;
            post.Description = sentPost.mt_excerpt;

            if (sentPost.mt_allow_comments == 1)
            {
                post.ExtensionInfo.IsCommentsEnabled = true;
            }
            else
            {
                post.ExtensionInfo.IsCommentsEnabled = false;
            }

            post.Categories.Clear();
            foreach (string item in sentPost.categories)
            {
                ItemCategory cat;
                ItemCategoryService catService = new ItemCategoryService();
                string slug = CommonUtils.TitleToSlug(item);
                cat = catService.GetBySlug(slug);

                if (cat != null)
                {
                    post.Categories.Add(cat);
                }
                else
                {
                    // Allowing new categories to be added.  (This breaks spec, but is supported via WLW)
                    ItemCategory newcat = ItemCategory.NewItemCategory();
                    newcat.Title = item;
                    newcat.Save();
                    post.Categories.Add(newcat);
                }
            }
            post.Tags.Clear();
            foreach (string item in sentPost.tags)
            {
                if (item != null && item.Trim() != string.Empty)
                {
                    ItemTagService tagService = new ItemTagService();
                    ItemTag tag;
                    tag = tagService.GetByTitle(item);
                    if (tag != null)
                    {
                        post.Tags.Add(tag);
                    }
                    else
                    {
                        tag = ItemTag.NewItemTag();
                        tag.Title = item;
                        post.Tags.Add(tag);
                    }
                }
            }
            

            if (sentPost.postDate != new DateTime())
            {
                //post.DateCreated = sentPost.postDate;
                post.DisplayStartTime = sentPost.postDate;
                post.CreateTime = DateTime.Now;
            }

            ItemPostService oService = new ItemPostService();
            oService.UpdateItemPost(post, userName, password);
            
            return post.Id.ToString();
        }

        /// <summary>
        /// metaWeblog.editPost
        /// </summary>
        /// <param name="postID">post guid in string format</param>
        /// <param name="userName">login username</param>
        /// <param name="password">login password</param>
        /// <param name="sentPost">struct with post details</param>
        /// <param name="publish">mark as published?</param>
        /// <returns>1 if successful</returns>
        internal bool EditPost(string postID, string userName, string password, MWAPost sentPost, bool publish)
        {
            ValidateRequest(userName, password);

            ItemPostService oService = new ItemPostService();
            
            User user = new UserService().GetByUserLogon(userName);
            //Guid gid = new Guid(postID);
            //ItemPost post = oService.GetById(gid, user.Id, user.UserRoleList);
            int id_number = Int32.Parse(postID);
            ItemPost post = oService.GetByIdNumber(id_number, user.Id, user.UserRoleList);

            //Post post = Post.GetPost(new Guid(postID));
            UserService userService = new UserService();

            if (String.IsNullOrEmpty(sentPost.wp_author))
            {
                User author = userService.GetByUserLogon(userName);
                if (author != null)
                {
                    post.Author = author;
                }
                //post.CreateUser = userName;
                post.UpdateUser = userName;
            }
            else
            {
                User author = userService.GetByUserLogon(sentPost.wp_author);
                if (author != null)
                {
                    post.Author = author;
                }
                //post.CreateUser = sentPost.author;
                post.UpdateUser = sentPost.wp_author;
            }
            post.Title = sentPost.title;
            post.Content = sentPost.description;
            //post.IsPublished = publish;
            post.Visible = publish;
            post.Slug = sentPost.wp_slug;
            post.Description = sentPost.mt_excerpt;

            if (sentPost.mt_allow_comments == 1)
            {
                post.ExtensionInfo.IsCommentsEnabled = true;
            }
            else
            {
                post.ExtensionInfo.IsCommentsEnabled = false;
            }

            post.Categories.Clear();
            foreach (string item in sentPost.categories)
            {
                ItemCategory cat;
                ItemCategoryService catService = new ItemCategoryService();
                string slug = CommonUtils.TitleToSlug(item);
                cat = catService.GetBySlug(slug);

                if (cat != null)
                {
                    post.Categories.Add(cat);
                }
                else
                {
                    // Allowing new categories to be added.  (This breaks spec, but is supported via WLW)
                    ItemCategory newcat = ItemCategory.NewItemCategory();
                    newcat.Title = item;
                    newcat.Save();
                    post.Categories.Add(newcat);
                }
            }
            post.Tags.Clear();
            foreach (string item in sentPost.tags)
            {
                if (item != null && item.Trim() != string.Empty)
                {
                    ItemTagService tagService = new ItemTagService();
                    ItemTag tag;
                    tag = tagService.GetByTitle(item);
                    if (tag != null)
                    {
                        post.Tags.Add(tag);
                    }
                    else
                    {
                        tag = ItemTag.NewItemTag();
                        tag.Title = item;
                        post.Tags.Add(tag);
                    }
                }
            }

            if (sentPost.postDate != new DateTime())
            {
                //post.DateCreated = sentPost.postDate.AddHours(-Data.SystemSettings.Instance.TimeZone);
                post.CreateTimeUtc = sentPost.postDate.ToUniversalTime();
            }


            oService.UpdateItemPost(post, userName, password);

            return true;
        }

        /// <summary>
        /// metaWeblog.getPost
        /// </summary>
        /// <param name="postID">post guid in string format</param>
        /// <param name="userName">login username</param>
        /// <param name="password">login password</param>
        /// <returns>struct with post details</returns>
        internal MWAPost GetPost(string postID, string userName, string password)
        {
            ValidateRequest(userName, password);

            MWAPost sendPost = new MWAPost();
            //Post post = Post.GetPost(new Guid(postID));

            ItemPostService oService = new ItemPostService();
            User user = new UserService().GetByUserLogon(userName);

            //Guid gid = new Guid(pageID);
            //ItemPost post = oService.GetById(gid, user.Id, user.UserRoleList);
            int id_number = Int32.Parse(postID);
            ItemPost post = oService.GetByIdNumber(id_number, user.Id, user.UserRoleList);

            sendPost.postID = post.Id.ToString();

            //sendPost.postDate = post.DateCreated;
            sendPost.postDate = post.DisplayStartTime;
            sendPost.title = post.Title;
            sendPost.description = post.Content;
            sendPost.link = post.AbsoluteLink.AbsoluteUri;
            sendPost.wp_slug = post.Slug;
            sendPost.mt_excerpt = post.Description;
            sendPost.publish = post.Visible; // post.IsPublished;

            sendPost.post_id_number = post.IdNumber;
            sendPost.post_guid = post.Id.ToString();
            sendPost.wp_author = post.Author.DisplayName;
            sendPost.wp_author_id = post.Author.IdNumber;
            sendPost.wp_author_display_name = post.Author.DisplayName;

            if (post.ExtensionInfo.IsCommentsEnabled)
            {
                sendPost.mt_allow_comments = 1;
            }
            else
            {
                sendPost.mt_allow_comments = 0;
            }
            

            List<string> cats = new List<string>();
            foreach (ItemCategory postCat in post.Categories)
            {
                cats.Add(postCat.Title);
            }
            sendPost.categories = cats;

            List<string> tags = new List<string>();
            foreach (ItemTag postTag in post.Tags)
            {
                tags.Add(postTag.Title);
            }
            sendPost.tags = tags;
           

            return sendPost;
        }

        /// <summary>
        /// metaWeblog.newMediaObject
        /// </summary>
        /// <param name="blogID">always 1000 in BlogEngine since it is a singlar blog instance</param>
        /// <param name="userName">login username</param>
        /// <param name="password">login password</param>
        /// <param name="mediaObject">struct with media details</param>
        /// <param name="request">The HTTP request.</param>
        /// <returns>struct with url to media</returns>
        internal MWAMediaInfo NewMediaObject(string blogID, string userName, string password, MWAMediaObject mediaObject, HttpContext request)
        {
            ValidateRequest(userName, password);

            MWAMediaInfo mediaInfo = new MWAMediaInfo();

            string rootPath = Data.SystemSettings.Instance.Settings("Path.Media");
            string serverPath = request.Server.MapPath(rootPath);
            string saveFolder = serverPath;
            string fileName = mediaObject.name;

            // Check/Create Folders & Fix fileName
            if (mediaObject.name.LastIndexOf('/') > -1)
            {
                saveFolder += mediaObject.name.Substring(0, mediaObject.name.LastIndexOf('/'));
                saveFolder = saveFolder.Replace('/', Path.DirectorySeparatorChar);
                fileName = mediaObject.name.Substring(mediaObject.name.LastIndexOf('/') + 1);
            }
            else
            {
                if (saveFolder.EndsWith(Path.DirectorySeparatorChar.ToString(), StringComparison.OrdinalIgnoreCase))
                    saveFolder = saveFolder.Substring(0, saveFolder.Length - 1);
            }
            if (!Directory.Exists(saveFolder))
                Directory.CreateDirectory(saveFolder);
            saveFolder += Path.DirectorySeparatorChar;

            // Save File
            FileStream fs = new FileStream(saveFolder + fileName, FileMode.Create);
            BinaryWriter bw = new BinaryWriter(fs);
            bw.Write(mediaObject.bits);
            bw.Close();

            // Set Url
            string rootUrl = CommonUtils.AbsoluteWebRoot.ToString();
            if (SystemSettings.Instance.RequireSSLMetaWeblogAPI)
            {
                rootUrl = rootUrl.Replace("https://", "http://");
            }

            string mediaType = mediaObject.type;
            if (mediaType.IndexOf('/') > -1)
                mediaType = mediaType.Substring(0, mediaType.IndexOf('/'));
            switch (mediaType)
            {
                case "image":
                case "notsent": // If there wasn't a type, let's pretend it is an image.
                    rootUrl += AccountSettings.Instance.PathImages;  //"image.axd?picture=";
                    break;
                default:
                    rootUrl += AccountSettings.Instance.PathFiles;  // "file.axd?file=";
                    break;
            }

            mediaInfo.url = rootUrl + mediaObject.name;
            return mediaInfo;
        }

        /// <summary>
        /// metaWeblog.getCategories
        /// </summary>
        /// <param name="blogID">always 1000 in BlogEngine since it is a singlar blog instance</param>
        /// <param name="userName">login username</param>
        /// <param name="password">login password</param>
        /// <param name="rootUrl">The root URL.</param>
        /// <returns>array of category structs</returns>
        internal List<MWACategory> GetCategories(string blogID, string userName, string password, string rootUrl)
        {
            List<MWACategory> categories = new List<MWACategory>();

            ValidateRequest(userName, password);

            ItemCategoryService catService = new ItemCategoryService();
            foreach (ItemCategory cat in catService.GetCollection())
            {
                MWACategory temp = new MWACategory();
                temp.title = cat.Title;
                temp.description = cat.Description;
                temp.htmlUrl = cat.AbsoluteLink.ToString();
                temp.rssUrl = cat.RssLink.ToString();
                categories.Add(temp);
            }
            
            return categories;
        }

        /// <summary>
        /// wp.getTags
        /// </summary>
        /// <param name="blogID"></param>
        /// <param name="userName"></param>
        /// <param name="password"></param>
        /// <returns>list of tags</returns>
        internal List<string> GetKeywords(string blogID, string userName, string password)
        {
            List<string> keywords = new List<string>();

            ValidateRequest(userName, password);

            bool useAllTags = true;



            // Algorithm 1: Use ALL the tags in the Tag Collection
            if (useAllTags)
            {
                ItemTagService tagService = new ItemTagService();
                ItemTagCollection tags = tagService.GetPublishedCollection();

                foreach (ItemTag tag in tags)
                {
                    keywords.Add(tag.Title);
                }
            }
            else
            {
                // Algorithm 2: Use ONLY the tags that appear in posts
                ItemPostCollection oPostCollection;
                ItemPostService oService = new ItemPostService();
                oPostCollection = oService.GetCollection();
                foreach (ItemPost post in oPostCollection)
                {
                    if (post.IsPublished)
                    {
                        foreach (ItemTag tag in post.Tags)
                        {
                            if (!keywords.Contains(tag.Title))
                            {
                                keywords.Add(tag.Title);
                            }
                        }
                    }
                }
            }
            keywords.Sort();

            return keywords;
        }

        /// <summary>
        /// metaWeblog.getRecentPosts
        /// </summary>
        /// <param name="blogID">always 1000 in BlogEngine since it is a singlar blog instance</param>
        /// <param name="userName">login username</param>
        /// <param name="password">login password</param>
        /// <param name="numberOfPosts">number of posts to return</param>
        /// <returns>array of post structs</returns>
        internal List<MWAPost> GetRecentPosts(string blogID, string userName, string password, int numberOfPosts)
        {
            ValidateRequest(userName, password);

            List<MWAPost> sendPosts = new List<MWAPost>();
            ItemPostCollection oPostCollection;
            ItemPostService oService = new ItemPostService();
            User user = new UserService().GetByUserLogon(userName);
            oPostCollection = oService.GetCollection(user.Id, user.UserRoleList, AccountSettings.Instance.AccountSiteDefaultLocationId);
            oPostCollection = oService.FilterItemPostsByIsPage(false, oPostCollection);

            //List<ItemPost> posts = Post.Posts;

            // Set End Point
            int stop = numberOfPosts;
            if (stop > oPostCollection.Count)
                stop = oPostCollection.Count;

            //foreach (ItemPost post in posts.GetRange(0, stop))
            int i = 0;
            foreach (ItemPost post in oPostCollection)
            {
                i++;
                if (i > numberOfPosts)
                {
                    break;
                }
                MWAPost tempPost = new MWAPost();
                List<string> tempCats = new List<string>();
                List<string> tempTags = new List<string>();

                tempPost.postID = post.Id.ToString();
                //tempPost.postDate = post.DateCreated;
                tempPost.wp_author = post.Author.DisplayName;
                tempPost.postDate = post.DisplayStartTime;
                tempPost.title = post.Title;
                tempPost.description = post.Content;
                tempPost.link = post.AbsoluteLink.AbsoluteUri;
                tempPost.wp_slug = post.Slug;
                tempPost.mt_excerpt = post.Description;

                if (post.ExtensionInfo.IsCommentsEnabled)
                {
                    tempPost.mt_allow_comments = 1;
                }
                else
                {
                    tempPost.mt_allow_comments = 0;
                }

                tempPost.publish = post.IsPublished;

                tempPost.post_id_number = post.IdNumber;
                tempPost.post_guid = post.Id.ToString();
                tempPost.wp_author = post.Author.DisplayName;
                tempPost.wp_author_id = post.Author.IdNumber;
                tempPost.wp_author_display_name = post.Author.DisplayName;

                List<string> cats = new List<string>();
                foreach (ItemCategory postCat in post.Categories)
                {
                    cats.Add(postCat.Title);
                }
                tempPost.categories = cats;

                List<string> tags = new List<string>();
                foreach (ItemTag postTag in post.Tags)
                {
                    tags.Add(postTag.Title);
                }
                tempPost.tags = tags;

                sendPosts.Add(tempPost);
                
            }

            return sendPosts;
        }

        /// <summary>
        /// blogger.getUsersBlogs
        /// </summary>
        /// <param name="appKey">Key from application.  Outdated methodology that has no use here.</param>
        /// <param name="userName">login username</param>
        /// <param name="password">login password</param>
        /// <param name="rootUrl">The root URL.</param>
        /// <returns>array of blog structs</returns>
        internal List<MWABlogInfo> GetUserBlogs(string appKey, string userName, string password, string rootUrl)
        {
            List<MWABlogInfo> blogs = new List<MWABlogInfo>();

            ValidateRequest(userName, password);

            MWABlogInfo temp = new MWABlogInfo();
            temp.url = rootUrl;
            temp.blogID = "1000";
            AccountSettings oAccountSettings = new AccountSettings();
            temp.blogName = oAccountSettings.AccountSiteName; // Data.AccountSettings.AccountSiteName;
            blogs.Add(temp);

            return blogs;
        }

        /// <summary>
        /// blogger.deletePost
        /// </summary>
        /// <param name="appKey">Key from application.  Outdated methodology that has no use here.</param>
        /// <param name="postID">post guid in string format</param>
        /// <param name="userName">login username</param>
        /// <param name="password">login password</param>
        /// <param name="publish">mark as published?</param>
        /// <returns></returns>
        internal bool DeletePost(string appKey, string postID, string userName, string password, bool publish)
        {
            ValidateRequest(userName, password);

            ItemPostService oService = new ItemPostService();
            
            try
            {
                Guid gid = new Guid(postID);
                ItemPost post = oService.GetById(gid); //Post.GetPost(new Guid(postID));
                //post.Delete();
                post.Deleted = true;
                oService.UpdateItemPost(post, userName, password);
                //post.Save(userName, password);
            }
            catch (Exception ex)
            {
                throw new MetaWeblogException("12", "DeletePost failed.  Error: " + ex.Message);
            }

            return true;
        }

        /// <summary>
        /// wp.newPage
        /// </summary>
        /// <param name="blogID">blogID in string format</param>
        /// <param name="userName">login username</param>
        /// <param name="password">login password</param>
        /// <param name="mPage"></param>
        /// <param name="publish"></param>
        /// <returns></returns>
        internal string NewPage(string blogID, string userName, string password, MWAPage mPage, bool publish)
        {
            ValidateRequest(userName, password);

            ItemPost page = ItemPost.NewItemPost(); // Page();
            page.Title = mPage.title;
            page.Content = mPage.description;
            page.Description = mPage.mt_excerpt; // Can not be set from WLW
            page.Keywords = mPage.mt_keywords;
            if (mPage.pageDate != new DateTime())
            {
                //page.CreateTime = mPage.pageDate;
                page.DisplayStartTime = mPage.pageDate;
            }
            UserService userService = new UserService();
            User author = userService.GetByUserLogon(userName);
            if (author != null)
            {
                page.Author = author;
            }
            //page.ShowInList = publish;
            //page.IsPublished = publish;
            page.Visible = publish;

            ItemPostService oService = new ItemPostService();
            if (mPage.wp_page_parent_id != 0)
            {
                ItemPost parent = oService.GetByIdNumber( mPage.wp_page_parent_id, author.Id, author.UserRoleList);
                page.ParentId = parent.Id;
            }

            if (mPage.mt_allow_comments == 1)
            {
                page.ExtensionInfo.IsCommentsEnabled = true;
            }
            else
            {
                page.ExtensionInfo.IsCommentsEnabled = false;
            }

            oService.UpdateItemPost(page, userName, password);
            //page.Save();

            return page.Id.ToString();
        }

        /// <summary>
        /// wp.getPages
        /// </summary>
        /// <param name="blogID">blogID in string format</param>
        /// <param name="userName">login username</param>
        /// <param name="password">login password</param>
        /// <returns></returns>
        internal List<MWAPage> GetPages(string blogID, string userName, string password)
        {
            ValidateRequest(userName, password);

            List<MWAPage> pages = new List<MWAPage>();
            ItemPostService oService = new ItemPostService();
            ItemPostCollection oCollection;
            User user = new UserService().GetByUserLogon(userName);
            oCollection = oService.GetCollection(user.Id, user.UserRoleList, AccountSettings.Instance.AccountSiteDefaultLocationId);
            oCollection = oService.FilterItemPostsByIsPage(true, oCollection);
            //oCollection = oService.GetItemPostsByPage(true);

            foreach (ItemPost page in oCollection)
            {
                MWAPage sendPage = new MWAPage();
                sendPage.pageID = page.Id.ToString();
                sendPage.title = page.Title;
                sendPage.description = page.Content;
                sendPage.mt_excerpt = page.Description;
                sendPage.mt_keywords = page.Keywords;
                //mPage.pageDate = page.DateCreated;
                sendPage.pageDate = page.DisplayStartTime;
                sendPage.link = page.AbsoluteLink.AbsoluteUri;
                sendPage.mt_convert_breaks = "__default__";

                sendPage.page_parent_guid = "";
                sendPage.wp_page_parent_id = 0;
                sendPage.wp_page_parent_title = "";
                if (page.ParentId != Guid.Empty)
                {
                    ItemPost parent = oService.GetById(page.ParentId, user.Id, user.UserRoleList);
                    string title = string.Empty;
                    if (parent != null)
                    {
                        sendPage.page_parent_guid = page.ParentId.ToString();
                        sendPage.wp_page_parent_id = parent.IdNumber;
                        sendPage.wp_page_parent_title = parent.Title;
                    }
                }

                sendPage.wp_slug = page.Slug;
                sendPage.publish = page.IsPublished;
                sendPage.wp_page_order = page.DisplayRank;
                sendPage.wp_author = page.Author.DisplayName;
                sendPage.wp_author_id = page.Author.IdNumber;
                sendPage.wp_author_display_name = page.Author.DisplayName;
                sendPage.page_id_number = page.IdNumber;
                sendPage.page_guid = page.Id.ToString();

                if (page.ExtensionInfo.IsCommentsEnabled)
                {
                    sendPage.mt_allow_comments = 1;
                }
                else
                {
                    sendPage.mt_allow_comments = 0;
                }

                pages.Add(sendPage);
            }

            return pages;
        }

        /// <summary>
        /// wp.getPage
        /// </summary>
        /// <param name="blogID">blogID in string format</param>
        /// <param name="pageID">page guid in string format</param>
        /// <param name="userName">login username</param>
        /// <param name="password">login password</param>
        /// <returns>struct with post details</returns>
        internal MWAPage GetPage(string blogID, string pageID, string userName, string password)
        {
            ValidateRequest(userName, password);

            MWAPage sendPage = new MWAPage();
            ItemPostService oService = new ItemPostService();

            User user = new UserService().GetByUserLogon(userName);
            //Guid gid = new Guid(pageID);
            //ItemPost page = oService.GetById(gid, user.Id, user.UserRoleList);
            int id_number = Int32.Parse(pageID);
            ItemPost page = oService.GetByIdNumber(id_number, user.Id, user.UserRoleList);

            sendPage.pageID = page.Id.ToString();
            sendPage.title = page.Title;
            sendPage.description = page.Content;
            sendPage.mt_excerpt = page.Description;
            sendPage.mt_keywords = page.Keywords;
            //sendPage.pageDate = page.DateCreated;
            sendPage.pageDate = page.DisplayStartTime;
            sendPage.link = page.AbsoluteLink.AbsoluteUri;
            sendPage.mt_convert_breaks = "__default__";

            sendPage.page_parent_guid = "";
            sendPage.wp_page_parent_id = 0;
            sendPage.wp_page_parent_title = "";
            if (page.ParentId != Guid.Empty)
            {
                ItemPost parent = oService.GetById(page.ParentId, user.Id, user.UserRoleList);
                string title = string.Empty;
                if (parent != null)
                {
                    sendPage.page_parent_guid = page.ParentId.ToString();
                    sendPage.wp_page_parent_id = parent.IdNumber;
                    sendPage.wp_page_parent_title = parent.Title;
                }
            }

            sendPage.wp_slug = page.Slug;
            sendPage.publish = page.IsPublished;
            sendPage.wp_page_order = page.DisplayRank;
            sendPage.wp_author = page.Author.DisplayName;
            sendPage.wp_author_id = page.Author.IdNumber;
            sendPage.wp_author_display_name = page.Author.DisplayName;
            sendPage.page_id_number = page.IdNumber;
            sendPage.page_guid = page.Id.ToString();

            if (page.ExtensionInfo.IsCommentsEnabled)
            {
                sendPage.mt_allow_comments = 1;
            }
            else
            {
                sendPage.mt_allow_comments = 0;
            }
            return sendPage;
        }

        internal bool EditPage(string blogID, string pageID, string userName, string password, MWAPage mPage, bool publish)
        {
            ValidateRequest(userName, password);

            //Page page = Page.GetPage(new Guid(pageID));
            ItemPostService oService = new ItemPostService();
            User user = new UserService().GetByUserLogon(userName);

            //Guid gid = new Guid(pageID);
            //ItemPost page = oService.GetById(gid, user.Id, user.UserRoleList);
            int id_number = Int32.Parse(pageID);
            ItemPost page = oService.GetByIdNumber(id_number, user.Id, user.UserRoleList);

            page.Title = mPage.title;
            page.Content = mPage.description;
            page.Description = mPage.mt_excerpt;
            page.Keywords = mPage.mt_keywords;
            page.ShowInList = publish;

            page.Visible = publish;
            //page.ShowInList = publish;

            UserService userService = new UserService();
            User author = userService.GetByUserLogon(userName);
            if (author != null)
            {
                page.Author = author;
            }

            if (mPage.wp_page_parent_id != 0)
            {
                ItemPost parent = oService.GetByIdNumber(mPage.wp_page_parent_id, author.Id, author.UserRoleList);
                page.ParentId = parent.Id;
            }

            if (mPage.mt_allow_comments == 1)
            {
                page.ExtensionInfo.IsCommentsEnabled = true;
            }
            else
            {
                page.ExtensionInfo.IsCommentsEnabled = false;
            }



            page.Slug = mPage.wp_slug;
            page.Visible = publish;
            page.DisplayRank = mPage.wp_page_order;

            //page.Save();
            //ItemPostService oService = new ItemPostService();
            //oService.SaveItemPost(page);
            oService.UpdateItemPost(page, userName, password);

            return true;
        }

        internal bool DeletePage(string blogID, string pageID, string userName, string password)
        {
            ValidateRequest(userName, password);
            ItemPostService oService = new ItemPostService();

            try
            {
                //Page page = Page.GetPage(new Guid(pageID));
                Guid gid = new Guid(pageID);
                ItemPost page = oService.GetById(gid);
                page.Deleted = true;
                oService.UpdateItemPost(page, userName, password);
            }
            catch (Exception ex)
            {
                throw new MetaWeblogException("15", "DeletePage failed.  Error: " + ex.Message);
            }

            return true;
        }

        internal List<MWAAuthor> GetAuthors(string blogID, string userName, string password)
        {
            ValidateRequest(userName, password);

            bool isAdminUser;
            //System.Web.Security.RoleProvider roleProvider = new CM.Core.Model.Security.CmdbRoleProvider();
            //isAdminUser = roleProvider.IsUserInRole(userName, AccountSettings.Instance.AccountRoleAdministrator);

            isAdminUser = Roles.IsUserInRole(userName, AccountSettings.Instance.AccountRoleAdministrator);

            bool isEditor = Roles.IsUserInRole(userName, AccountSettings.Instance.AccountRoleEditor);
            
            List<MWAAuthor> authors = new List<MWAAuthor>();
            UserService userService = new UserService();

            if ((isAdminUser) || (isEditor))
            {
                int total = 0;
                int count = 0;
                MembershipUserCollection users = Membership.Provider.GetAllUsers(0, 999, out total);
                

                foreach (MembershipUser user in users)
                {
                    count++;
                    MWAAuthor temp = new MWAAuthor();
                    User cmuser = userService.GetByUserName(user.UserName);
                    temp.user_id = user.UserName;
                    temp.user_idnumber = cmuser.IdNumber;
                    temp.user_id_guid = cmuser.Id.ToString();
                    
                    if (cmuser != null)
                    {
                        temp.display_name = cmuser.DisplayName;
                        temp.user_login = cmuser.UserLogon;
                    }
                    else
                    {
                        temp.user_login = user.UserName;
                        temp.display_name = user.UserName;
                    }
                    temp.user_email = user.Email;
                    temp.meta_value = "";

                    bool isAuthor = Roles.IsUserInRole(user.UserName, AccountSettings.Instance.AccountRoleAuthor);
                    isEditor = Roles.IsUserInRole(user.UserName, AccountSettings.Instance.AccountRoleEditor);
                    if (isAuthor || isEditor)
                    {
                        authors.Add(temp);
                    }
                }
            }
            else
            {
                // If not admin, just add that user to the options.
                MembershipUser user = Membership.GetUser(userName);
                MWAAuthor temp = new MWAAuthor();
                User cmuser = userService.GetByUserName(user.UserName);
                temp.user_id = user.UserName;
                temp.user_idnumber = cmuser.IdNumber;
                temp.user_id_guid = cmuser.Id.ToString();

                if (cmuser != null)
                {
                    temp.display_name = cmuser.DisplayName;
                    temp.user_login = cmuser.UserLogon;
                }
                else
                {
                    temp.user_login = user.UserName;
                    temp.display_name = user.UserName;
                }
                temp.user_email = user.Email;
                temp.meta_value = "";
                authors.Add(temp);
            }
            return authors;
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Checks username and password.  Throws error if validation fails.
        /// </summary>
        /// <param name="userName"></param>
        /// <param name="password"></param>
        private void ValidateRequest(string userName, string password)
        {
            if (!Membership.ValidateUser(userName, password))
            {
                throw new MetaWeblogException("11", "User authentication failed");
            }
            //ItemPostService oService = new ItemPostService();
            //oService.Clear();
            //if (!FormsAuthentication.Authenticate(userName, password))
            //{
            //    throw new MetaWeblogException("11", "User authentication failed");
            //}
            FormsAuthentication.SetAuthCookie(userName, false);
        }

        /// <summary>
        /// Returns Category Guid from Category name.
        /// </summary>
        /// <remarks>
        /// Reverse dictionary lookups are ugly.
        /// </remarks>
        /// <param name="name"></param>
        /// <param name="cat"></param>
        /// <returns></returns>
        private bool LookupCategoryGuidByName(string name, out ItemCategory cat)
        {
            cat = new ItemCategory();
            ItemCategoryService oService = new ItemCategoryService();
            ItemCategoryCollection oCollection;
            oCollection = oService.GetCollection();
            foreach (ItemCategory item in oCollection)
            {
                if (item.Title == name)
                {
                    cat = item;
                    return true;
                }
            }
            return false;
        }

        #endregion
    }

    /// <summary>
    /// Exception specifically for MetaWeblog API.  Error (or fault) responses 
    /// request a code value.  This is our chance to add one to the exceptions
    /// which can be used to produce a proper fault.
    /// </summary>
    [Serializable()]
    public class MetaWeblogException : Exception
    {
        /// <summary>
        /// Constructor to load properties
        /// </summary>
        /// <param name="code">Fault code to be returned in Fault Response</param>
        /// <param name="message">Message to be returned in Fault Response</param>
        public MetaWeblogException(string code, string message)
            : base(message)
        {
            _code = code;
        }

        private string _code;
        /// <summary>
        /// Code is actually for Fault Code.  It will be passed back in the 
        /// response along with the error message.
        /// </summary>
        public string Code
        {
            get
            {
                return _code;
            }
        }
    }

    /// <summary>
    /// MetaWeblog Category struct
    /// returned as an array from GetCategories
    /// </summary>
    internal struct MWACategory
    {
        /// <summary>
        /// Category title
        /// </summary>
        public string description;
        /// <summary>
        /// Url to thml display of category
        /// </summary>
        public string htmlUrl;
        /// <summary>
        /// Url to RSS for category
        /// </summary>
        public string rssUrl;
        /// <summary>
        /// The guid of the category
        /// </summary>
        public string id;
        /// <summary>
        /// The title/name of the category
        /// </summary>
        public string title;
    }

    /// <summary>
    /// MetaWeblog BlogInfo struct
    /// returned as an array from getUserBlogs
    /// </summary>
    internal struct MWABlogInfo
    {
        /// <summary>
        /// Blog Url
        /// </summary>
        public string url;
        /// <summary>
        /// Blog ID (Since BlogEngine.NET is single instance this number is always 10.
        /// </summary>
        public string blogID;
        /// <summary>
        /// Blog Title
        /// </summary>
        public string blogName;
    }

    /// <summary>
    /// MetaWeblog Fault struct
    /// returned when error occurs
    /// </summary>
    internal struct MWAFault
    {
        /// <summary>
        /// Error code of Fault Response
        /// </summary>
        public string faultCode;
        /// <summary>
        /// Message of Fault Response
        /// </summary>
        public string faultString;
    }

    /// <summary>
    /// MetaWeblog MediaObject struct
    /// passed in the newMediaObject call
    /// </summary>
    internal struct MWAMediaObject
    {
        /// <summary>
        /// Name of media object (filename)
        /// </summary>
        public string name;
        /// <summary>
        /// Type of file
        /// </summary>
        public string type;
        /// <summary>
        /// Media
        /// </summary>
        public byte[] bits;
    }

    /// <summary>
    /// MetaWeblog MediaInfo struct
    /// returned from NewMediaObject call
    /// </summary>
    internal struct MWAMediaInfo
    {
        /// <summary>
        /// Url that points to Saved MediaObejct
        /// </summary>
        public string url;
    }

    /// <summary>
    /// MetaWeblog Post struct
    /// used in newPost, editPost, getPost, recentPosts
    /// not all properties are used everytime.
    /// </summary>
    internal struct MWAPost
    {
        /// <summary>
        /// PostID Guid in string format
        /// </summary
        public string postID;

        public int post_id_number;

        public string post_guid;
        /// <summary>
        /// Title of Blog Post
        /// </summary>
        public string title;
        /// <summary>
        /// Link to Blog Post
        /// </summary>
        public string link;
        /// <summary>
        /// Content of Blog Post
        /// </summary>
        public string description;
        /// <summary>
        /// List of Categories assigned for Blog Post
        /// </summary>
        public List<string> categories;
        /// <summary>
        /// List of Tags assinged for Blog Post
        /// </summary>
        public List<string> tags;
        /// <summary>
        /// Display date of Blog Post (DateCreated)
        /// </summary>
        public DateTime postDate;
        /// <summary>
        /// Whether the Post is published or not.
        /// </summary>
        public bool publish;
        /// <summary>
        /// Slug of post
        /// </summary>
        public string wp_slug;
        /// <summary>
        /// CommentPolicy (Allow/Deny)
        /// </summary>
        //public string mt_allow_comments;
        public int mt_allow_comments; // WP value is an int.
        /// <summary>
        /// Excerpt
        /// </summary>
        public string mt_excerpt;
        /// <summary>
        /// wp_author_id
        /// </summary>
        public string wp_author;

        public int wp_author_id;

        public string wp_author_display_name;

        public string mt_keywords;

    }

    ///// <summary>
    ///// MetaWeblog UserInfo struct
    ///// returned from GetUserInfo call
    ///// </summary>
    ///// <remarks>
    ///// Not used currently, but here for completeness.
    ///// </remarks>
    //internal struct MWAUserInfo
    //{
    //  /// <summary>
    //  /// User Name Proper
    //  /// </summary>
    //  //public string nickname;
    //  /// <summary>
    //  /// Login ID
    //  /// </summary>
    //  //public string userID;
    //  /// <summary>
    //  /// Url to User Blog?
    //  /// </summary>
    //  //public string url;
    //  /// <summary>
    //  /// Email address of User
    //  /// </summary>
    //  public string email;
    //  /// <summary>
    //  /// User LastName
    //  /// </summary>
    //  public string lastName;
    //  /// <summary>
    //  /// User First Name
    //  /// </summary>
    //  public string firstName;
    //}

    /// <summary>
    /// wp Page Struct
    /// </summary>
    internal struct MWAPage
    {
        /// <summary>
        /// PostID Guid in string format
        /// </summary>
        public string pageID;

        public int page_id_number;

        public string page_guid;

        /// <summary>
        /// Title of Blog Post
        /// </summary>
        public string title;
        /// <summary>
        /// Link to Blog Post
        /// </summary>
        public string link;
        /// <summary>
        /// Content of Blog Post
        /// </summary>
        public string description;

        public string mt_excerpt;
        /// <summary>
        /// Display date of Blog Post (DateCreated)
        /// </summary>
        public DateTime pageDate;
        /// <summary>
        /// Convert Breaks
        /// </summary>        
        public string mt_convert_breaks;
        /// <summary>
        /// Page Parent ID
        /// </summary>
        public int wp_page_parent_id;

        public int mt_allow_comments; // WP value is an int.

        public string wp_page_parent_title;

        public string page_parent_guid;
        /// <summary>
        /// Page keywords
        /// </summary>
        public string mt_keywords;

        public string wp_slug;

        public bool publish;

        public string wp_author;

        public int wp_author_id;

        public string wp_author_display_name;

        public int wp_page_order;
    }

    /// <summary>
    /// wp Author struct
    /// </summary>
    internal struct MWAAuthor
    {
        /// <summary>
        /// userID - Specs call for a int, but our ID is a string.
        /// </summary>
        public string user_id;
        /// <summary>
        /// user login name
        /// </summary>
        public int user_idnumber;

        public string user_id_guid;

        public string user_login;
        /// <summary>
        /// display name
        /// </summary>
        public string display_name;
        /// <summary>
        /// user email
        /// </summary>
        public string user_email;
        /// <summary>
        /// nothing to see here.
        /// </summary>
        public string meta_value;
    }

}
